@@ -2,6 +2,7 @@
 
 import abc
 import collections.abc
+from collections import OrderedDict
 import enum
 import typing
 from rpy2.rinterface_lib import embedded
@@ -239,15 +240,14 @@ class NULLType(Sexp, metaclass=SingletonABC):
         return self._sexpobject.rid
 
 
-# TODO: Duplicate declaration in R_API.h ?
 class CETYPE(enum.Enum):
     """Character encodings for R string."""
-    CE_NATIVE = 0
-    CE_UTF8 = 1
-    CE_LATIN1 = 2
-    CE_BYTES = 3
-    CE_SYMBOL = 5
-    CE_ANY = 99
+    CE_NATIVE = openrlib.rlib.CE_NATIVE
+    CE_UTF8 = openrlib.rlib.CE_UTF8
+    CE_LATIN1 = openrlib.rlib.CE_LATIN1
+    CE_BYTES = openrlib.rlib.CE_BYTES
+    CE_SYMBOL = openrlib.rlib.CE_SYMBOL
+    CE_ANY = openrlib.rlib.CE_ANY
 
 
 class NCHAR_TYPE(enum.Enum):
@@ -313,8 +313,9 @@ class SexpEnvironment(Sexp):
         elif not len(key):
             raise ValueError('The key must be a non-empty string.')
         with memorymanagement.rmemory() as rmemory:
+            key_cchar = conversion._str_to_cchar(key, 'utf-8')
             symbol = rmemory.protect(
-                openrlib.rlib.Rf_install(conversion._str_to_cchar(key))
+                openrlib.rlib.Rf_install(key_cchar)
             )
             if wantfun:
                 # One would expect this to be like
@@ -323,8 +324,11 @@ class SexpEnvironment(Sexp):
                 # the environment. :/
                 rho = self
                 while rho.rid != emptyenv.rid:
-                    res = _rinterface._findVarInFrame(symbol,
-                                                      rho.__sexp__._cdata)
+                    res = rmemory.protect(
+                        _rinterface.findvar_in_frame_wrap(
+                            rho.__sexp__._cdata, symbol
+                        )
+                    )
                     if _rinterface._TYPEOF(res) in (openrlib.rlib.CLOSXP,
                                                     openrlib.rlib.BUILTINSXP):
                         break
@@ -346,10 +350,16 @@ class SexpEnvironment(Sexp):
         elif not len(key):
             raise ValueError('The key must be a non-empty string.')
         with memorymanagement.rmemory() as rmemory:
+            key_cchar = conversion._str_to_cchar(key)
             symbol = rmemory.protect(
-                openrlib.rlib.Rf_install(conversion._str_to_cchar(key))
+                openrlib.rlib.Rf_install(key_cchar)
             )
-            res = _rinterface._findVarInFrame(symbol, self.__sexp__._cdata)
+            res = rmemory.protect(
+                _rinterface.findvar_in_frame_wrap(
+                    self.__sexp__._cdata, symbol
+                )
+            )
+
         # TODO: move check of R_UnboundValue to _rinterface
         if res == openrlib.rlib.R_UnboundValue:
             raise KeyError("'%s' not found" % key)
@@ -367,8 +377,9 @@ class SexpEnvironment(Sexp):
                              'empty environments.')
         # TODO: call to Rf_duplicate needed ?
         with memorymanagement.rmemory() as rmemory:
+            key_cchar = conversion._str_to_cchar(key)
             symbol = rmemory.protect(
-                openrlib.rlib.Rf_install(conversion._str_to_cchar(key))
+                openrlib.rlib.Rf_install(key_cchar)
             )
             cdata = rmemory.protect(conversion._get_cdata(value))
             cdata_copy = rmemory.protect(
@@ -741,20 +752,60 @@ class StrSexpVector(SexpVector):
         )
 
 
-# TODO: complete class names.
-_DEFAULT_RCLASS_NAMES = {
+class RVersion(metaclass=Singleton):
+
+    _version = None
+
+    def __init__(self):
+        assert embedded.isinitialized()
+        robj = StrSexpVector(['R.version'])
+        with memorymanagement.rmemory() as rmemory:
+            parsed = _rinterface._parse(robj.__sexp__._cdata, 1, rmemory)
+        res = baseenv['eval'](parsed)
+        self._version = OrderedDict((k, v[0]) for k, v in zip(res.names, res))
+
+    def __getitem__(self, k):
+        return self._version[k]
+
+    def keys(self):
+        return self._version.keys()
+
+
+_TYPE2STR = {
+    RTYPES.NILSXP: 'NULL',
+    RTYPES.SYMSXP: 'symbol',  # alias: name
+    RTYPES.LISTSXP: 'pairlist',
+    RTYPES.CLOSXP: 'closure',
     RTYPES.ENVSXP: 'environment',
-    RTYPES.CLOSXP: 'function',
-    RTYPES.SPECIALSXP: 'function',
-    RTYPES.BUILTINSXP: 'function',
-    RTYPES.REALSXP: 'numeric',
+    RTYPES.PROMSXP: 'promise',
+    RTYPES.LANGSXP: 'language',
+    RTYPES.SPECIALSXP: 'special',
+    RTYPES.BUILTINSXP: 'builtin',
+    RTYPES.CHARSXP: 'char',
+    RTYPES.LGLSXP: 'logical',
+    RTYPES.INTSXP: 'integer',
+    RTYPES.REALSXP: 'double',  # alias: numeric
+    RTYPES.CPLXSXP: 'complex',
     RTYPES.STRSXP: 'character',
-    RTYPES.SYMSXP: 'name',
+    RTYPES.DOTSXP: '...',
+    RTYPES.ANYSXP: 'any',
+    RTYPES.EXPRSXP: 'expression',
     RTYPES.VECSXP: 'list',
-    RTYPES.LANGSXP: 'language'}
+    RTYPES.EXTPTRSXP: 'externalptr',
+    RTYPES.BCODESXP: 'bytecode',
+    RTYPES.WEAKREFSXP: 'weakref',
+    RTYPES.RAWSXP: 'raw',
+    RTYPES.S4SXP: 'S4'
+}
 
 
 def rclass_get(scaps: _rinterface.CapsuleBase) -> StrSexpVector:
+    """ Get the R class name.
+
+    If no specific attribute "class" is defined from the objects, this
+    will perform the equivalent of R_data_class()
+    (src/main/attrib.c in the R source code).
+    """
     rlib = openrlib.rlib
     with memorymanagement.rmemory() as rmemory:
         classes = rmemory.protect(
@@ -767,15 +818,41 @@ def rclass_get(scaps: _rinterface.CapsuleBase) -> StrSexpVector:
             ndim = rlib.Rf_length(dim)
             if ndim > 0:
                 if ndim == 2:
-                    classname = 'matrix'
+                    if int(RVersion()['major']) >= 4:
+                        classname = ('matrix', 'array')
+                    else:
+                        classname = ('matrix', )
                 else:
-                    classname = 'array'
+                    classname = ('array', )
             else:
                 typeof = RTYPES(scaps.typeof)
-                classname = _DEFAULT_RCLASS_NAMES.get(
-                    typeof, str(typeof))
-            classes = StrSexpVector.from_iterable(
-                [classname])
+                if typeof in (RTYPES.CLOSXP,
+                              RTYPES.SPECIALSXP,
+                              RTYPES.BUILTINSXP):
+                    classname = ('function', )
+                elif typeof == RTYPES.REALSXP:
+                    classname = ('numeric', )
+                elif typeof == RTYPES.SYMSXP:
+                    classname = ('name', )
+                elif typeof == RTYPES.LANGSXP:
+                    symb = rlib.CAR(scaps._cdata)
+                    if openrlib.rlib.Rf_isSymbol(symb):
+                        symb_rstr = openrlib.rlib.PRINTNAME(symb)
+                        symb_str = conversion._cchar_to_str(
+                            openrlib.rlib.R_CHAR(symb_rstr),
+                            conversion._R_ENC_PY[openrlib.rlib
+                                                 .Rf_getCharCE(symb_rstr)]
+                        )
+                        if symb_str in ('if', 'while', 'for', '=',
+                                        '<-', '(', '{'):
+                            classname = (symb_str, )
+                        else:
+                            classname = ('call', )
+                    else:
+                        classname = ('call', )
+                else:
+                    classname = (_TYPE2STR.get(typeof, str(typeof)), )
+            classes = StrSexpVector.from_iterable(classname)
         else:
             classes = conversion._cdata_to_rinterface(classes)
     return classes
@@ -785,6 +862,10 @@ def rclass_set(
         scaps: _rinterface.CapsuleBase,
         value: 'typing.Union[StrSexpVector, str]'
 ) -> None:
+    """ Set the R class.
+
+    :param:`scaps` A capsule with a pointer to an R object.
+    :param:`value` An R vector of strings."""
     if isinstance(value, StrSexpVector):
         value_r = value
     elif isinstance(value, str):
